function images = pco_adaptor_example_properties(triggermode, ...
    binning_hor, binning_vert, conv_factor, exptime, pixelclock, ...
    imcount, timeout_s)
% This function shows the general and device specific properties and sets 
% some of the general and also device properties. At the end an image
% series is recorded and shown. Depending on the trigger type the image
% recording either starts directly or has to be triggered by a keypress
% event.
%
%   [images] = pco_adaptor_example(triggermode, ...
%       binning_hor, binning_vert, conv_factor, exptime, pixelclock, ...
%       imcount)
%
% * Input parameters
%       triggermode:    Trigger mode to be set ('immediate' or 'manual')
%       binning_hor:    Horizontal binning
%       binning_vert:   Vertical binning
%       conv_factor:    Conversion factor
%       exptime:        Exposure time in [s]
%       pixelclock:     Pixelclock
%       imcount         Number of images to acquire
%       timeout_s:      Number of seconds to wait before video 
%                       input timeout occurs
%
% * Output parameters
%       images:         Stack with acquired images
%
% - Display information about the adaptor and connected devices
% - Get current settings and available properties that can be set
% - Set image count and trigger mode
% - Set camera specific device properties
% - Start preview, acquire images, stop preview
% - Display the acquired images and reset the adaptor
%

%% Set default values for paramters that haven't been set
if(~exist('triggermode','var'))
    triggermode = 'immediate';
end

if(~exist('binning_hor','var'))
    binning_hor = '01';
end

if(~exist('binning_vert','var'))
    binning_vert = '01';
end

if(~exist('conv_factor','var'))
    conv_factor = 0;
end

if(~exist('exptime','var'))
    exptime = 0.005;
end

if(~exist('pixelclock','var'))
    pixelclock = 0;
end

if(~exist('imcount','var'))
    imcount = 10;
end

if(~exist('timeout_s','var'))
    timeout_s = 120;
end

%Identify required adaptor
if verLessThan('matlab','9.0') %older than R2016a
  error('This adaptor is supported in Matlab R2016a and later versions'); 
else %R2016a and newer
  if(strcmp(computer('arch'),'win64'))
    adaptorName = ['pcocameraadaptor_r' version('-release')];
  elseif(strcmp(computer('arch'),'glnxa64'))
    adaptorName = ['pcocameraadaptor_r' version('-release')];
  else
    error('This platform is not supported.');
  end
end

%% Configure adaptor and display imaq information
%Get information about installed adaptors
imaqhwinfo

%Get information about the PCOCameraAdaptor
imaqhwinfo(adaptorName)

%Get information about the first device of the adaptor
imaqhwinfo(adaptorName,0)

%Create video input object
vid = videoinput(adaptorName, 0);

%Create adaptor source
src = getselectedsource(vid);

%% Show and set genaral properties
%Show current adaptor settings
get(vid)

%Show all possible general property set options
set(vid)

%Set logging to memory
vid.LoggingMode = 'memory';

%Set the timeout property of video input object 'vid'
set(vid,'Timeout',timeout_s);

%Set trigger mode
triggerconfig(vid, triggermode);

%Set frames per trigger
vid.FramesPerTrigger = imcount;

%% Show and set device properties
%Show current camera settings
get(src)

%Show all possible device property set options
set(src)

%Set horizontal binnig
src.B1BinningHorizontal = binning_hor;

%Set vertical binning
src.B2BinningVertical = binning_vert;

%Set conversion factor if specified
if conv_factor ~= 0
src.CFConversionFactor_e_count = conv_factor;
end

%Set exposure time
src.ExposureTime_s = exptime;

%Set Pixelclock if specified
if pixelclock ~=0
src.PCPixelclock_Hz = pixelclock;
end

%Set timestamp mode
src.TMTimestampMode = 'BinaryAndAscii';

%% Record images
%Start preview
preview(vid);

%Start acquisition
start(vid);

if strcmp(triggermode,'manual')
    %Wait for keypress
    pause;
    
    %Force trigger command
    trigger(vid);
end

%Read out all images and remove them from memory buffer
%Function blocks until the images have been aquired
images = getdata(vid);

%Stop preview
stoppreview(vid);

%% Show images
%Show images in Video Viewer app
implay(images);

%Reset adaptor
imaqreset;