function imgStack = flim_rearrange_stack(imgStackRaw, phaseNumber, ...
    addPhaseSampling, phaseOrder, tapSelection, asymCorrection)

%
% function imgStack = flim_rearrange_stack(imgStackRaw, phaseNumber, ...
%     addPhaseSampling, phaseOrder, tapSelection, asymCorrection)
%
% This function rearranges a raw image stack recorded with the pco.flim
% camera into an image stack of linearly ascending phases for further
% calculations.
% Note: Not all combinations of input parameters are meaningful thus being
% not implemented.
%
% Input parameters:
%
% imgStackRaw:      3D image stack recorded with the pco.flim camera
% phaseNumber:      Selects the number of equidistant phase samples per
%                   modulation period of 360 degrees (2, 4, 8 or 16).
%                   Additionally, it offers a 'shiftable pair' mode in
%                   which taps A and B can be manually phase-shifted
%                   relative to the modulation signal.
%                   Type string: 'shiftablePair', '02', '04', '08', '16'
% addPhaseSampling: If set to 'yes' all phase samples are sampled by both
%                   taps A and B, doubling the total number of phase
%                   images. It is used for asymmetry correction of the
%                   taps.
%                   Type string: 'no', 'yes'
% phaseOrder:       Determines the arrangement of adjacent phase pairs. If
%                   'opposite' is selected all complementary phase pairs
%                   are sorted adjacently. If 'ascending' is selected the
%                   phase pairs cover the modulation period linearly. This
%                   selection is meaningful if 'Additional Phase Sampling'
%                   is set to 'yes'.
%                   Type string: 'ascending', 'opposite'
% tapSelection:     Determines which taps are read (Tap A & B, Tap A or
%                   Tap B).
%                   Type string: 'Tap A', 'Tap B', 'Tap A+B'
% asymCorrection:   If switched 'on' the asymmetry correction is performed
%                   within the camera by averaging corresponding phase
%                   images of taps A and B.
%                   This mode requires the following settings:
%                   - phaseNumber:      all except 'shiftablePair'
%                   - addPhaseSampling: 'yes',
%                   - phaseOrder:       'opposite'
%                   - tapSelection:     'Tap A+B'
%                   Type string: 'on', 'off'
%
% Return value:
%
% imgStack: 3D image stack of linearly ascending phases
%

    % evaluate type of phase sorting
    sorting = '';

    if ( ...
        strcmp(addPhaseSampling, 'no') && ...
        strcmp(tapSelection, 'Tap A+B') && ...
        strcmp(asymCorrection, 'off') ...
    )
        sorting = 'ALTERNATING';
    elseif ( ...
        strcmp(addPhaseSampling, 'yes') && ...
        strcmp(phaseOrder, 'ascending') && ...
        ( ...
            strcmp(tapSelection, 'Tap A') || ...
            strcmp(tapSelection, 'Tap B') ...
        ) && ...
        strcmp(asymCorrection, 'off') ...
    )
        sorting = 'ASCENDING';
    elseif ( ...
        strcmp(addPhaseSampling, 'yes') && ...
        strcmp(phaseOrder, 'ascending') && ...
        strcmp(tapSelection, 'Tap A+B') && ...
        strcmp(asymCorrection, 'off') ...
    )
        sorting = 'EXTERNAL_AVERAGING';
    elseif ( ...
        strcmp(addPhaseSampling, 'yes') && ...
        strcmp(phaseOrder, 'opposite') && ...
        strcmp(tapSelection, 'Tap A+B') && ...
        strcmp(asymCorrection, 'off') ...
    )
        sorting = 'EXTERNAL_AVERAGING_OPPOSITE';
    elseif ( ...
        strcmp(addPhaseSampling, 'yes') && ...
        strcmp(phaseOrder, 'opposite') && ...
        strcmp(tapSelection, 'Tap A+B') && ...
        strcmp(asymCorrection, 'on') ...
    )
        sorting = 'ALTERNATING';
    elseif ( ...
        strcmp(addPhaseSampling, 'yes') && ...
        strcmp(phaseOrder, 'opposite') && ...
        ( ...
            strcmp(tapSelection, 'Tap A') || ...
            strcmp(tapSelection, 'Tap B') ...
        ) && ...
        strcmp(asymCorrection, 'off') ...
    )
        sorting = 'ALTERNATING';
    else
        error('Invalid combination of input parameters!');
        return
    end

    % get number of equidistant phases
    if (strcmp(phaseNumber, 'shiftablePair') || strcmp(phaseNumber, '02'))
        nPhases = 2;
    elseif (strcmp(phaseNumber, '04'))
        nPhases = 4;
    elseif (strcmp(phaseNumber, '08'))
        nPhases = 8;
    elseif (strcmp(phaseNumber, '16'))
        nPhases = 16;
    else
        error('Invalid number of phases!');
        return
    end

    % rearrange phase image stack
    imgStack = zeros(size(imgStackRaw, 1), size(imgStackRaw, 2), nPhases);

    if (strcmp(sorting, 'ASCENDING'))
        imgStack = imgStackRaw;
    elseif (strcmp(sorting, 'ALTERNATING'))
        imgStack = imgStackRaw(:, :, [1:2:nPhases 2:2:nPhases]);
    elseif (strcmp(sorting, 'EXTERNAL_AVERAGING'))
        imgStack = (imgStackRaw(:, :, 1:2:2*nPhases) + ...
                    imgStackRaw(:, :, [nPhases+2:2:2*nPhases ...
                    2:2:nPhases])) / 2;
    elseif (strcmp(sorting, 'EXTERNAL_AVERAGING_OPPOSITE'))
        imgStack = (imgStackRaw(:, :, [1:4:2*nPhases 3:4:2*nPhases]) + ...
                    imgStackRaw(:, :, [4:4:2*nPhases 2:4:2*nPhases])) / 2;
    else
        error('Unknown phase sorting!');
        return
    end

end
